# References
#
# The reference is for making input/file dialogs.
#
# Author: chanmaru
# Date(accessed): 2020/3/21
# Availability: https://qiita.com/chanmaru/items/1b64aa91dcd45ad91540

import numpy as np
import tkinter as tk
import copy, random, os
from tkinter import ttk,filedialog,simpledialog

class Agent:
    name = ""
    prefs = []
    def __init__(self, name):
        self.name = name
        self.prefs = []

class Student(Agent):
    pass

class Faculty(Agent):
    quota = 0
    def set_quota(self, quota):
        self.quota = quota

class Department:
    l_faculty = []
    l_students = []
    def __init__(self,n_faculty,n_students):
        for i in range(n_faculty):
            self.l_faculty.append(Faculty("t" + str(i)))

        for i in range(n_students):
            self.l_students.append(Student("s" + str(i)))

        for i in self.l_faculty:
            print(i.name)

    def generate_prefs(self,avg_list_len=10,avg_quota=5):
        # generate random student preferences
        x = np.random.poisson(lam=avg_list_len, size=len(self.l_students))
        for stu in self.l_students:
            stu_index = self.l_students.index(stu)
            if x[stu_index] > len(self.l_faculty):
                x[stu_index] = len(self.l_faculty)
            set_faculty = random.sample(self.l_faculty, x[stu_index])
            random.shuffle(set_faculty)
            stu.prefs = set_faculty
            print(stu.name + ',' + ','.join([f.name for f in stu.prefs]))
            for f in set_faculty:
                print(f.name)
                f.prefs.append(stu)
                print(f.name + ',' + ','.join([s.name for s in f.prefs]))

        # generate random faculty preferences
        for f in self.l_faculty:
            f.set_quota(avg_quota)
            print(f.name + ',' + ','.join([s.name for s in f.prefs]))
            random.shuffle(f.prefs)
            print(f.name + ',' + ','.join([s.name for s in f.prefs]))

    def get_students(self):
        return self.l_students

    def get_faculty(self):
        return self.l_faculty

# default values
n_faculty = 10
n_students = 50
avg_list_len = 10
avg_quota = 5

max_faculty = 1000
max_students = 10000

#
# 入力ダイアログ表示- 数値入力、次に保存するファイル名入力
#
root = tk.Tk()
root.withdraw()

msg = "配属マッチング用擬似データ生成:\n"
answer = simpledialog.askinteger("Input", msg+"教員の数は? (最大: {})".format(max_faculty),
                                parent=root,minvalue=1, maxvalue=1000)
# use the default value if answer is None.
if answer is not None:
    n_faculty = answer

answer = simpledialog.askinteger("Input", msg+"学生の数は?  (最大: {})".format(max_students),
                                 parent=root,minvalue=0, maxvalue=10000)
# use the default value if answer is None.
if answer is not None:
    n_students = answer

answer = simpledialog.askinteger("Input", msg+"教員のゼミ/研究室の定員は？",
                               parent=root,minvalue=0, maxvalue=n_students)
# use the default value if answer is None.
if answer is not None:
    avg_quota = answer

answer = simpledialog.askfloat("Input", msg+"学生の希望登録数（平均値）？",
                               parent=root,minvalue=0.0, maxvalue=n_faculty)
# use the default value if answer is None.
if answer is not None:
    avg_list_len = answer


# ファイル選択ダイアログの表示
root = tk.Tk()
root.withdraw()
fTyp = [("","*.csv")]
iDir = os.path.abspath(os.path.dirname(__file__))
file_out_fac = tk.filedialog.asksaveasfilename(filetypes = fTyp,initialdir = iDir, \
        title="生成した教員の優先順位情報を保存するファイルを選択", defaultextension=".csv")
file_out_stu = tk.filedialog.asksaveasfilename(filetypes = fTyp,initialdir = iDir, \
        title="生成した学生の希望順位情報を保存するファイルを選択", defaultextension=".csv")
if file_out_fac == '':
    file_out_fac = 'test_faculty.csv'
if file_out_stu == '':
    file_out_stu = 'test_students.csv'

dept = Department(n_faculty,n_students)
dept.generate_prefs(avg_list_len,avg_quota)
students = dept.get_students()
faculty = dept.get_faculty()

# create a file for faculty preferences
with open(file_out_fac, 'wt') as f:
    f.write('id,,q0,q2,type' + ','* (n_students+1) + '\n')
    for fac in faculty:
        str_fac = ",{},{},,".format(fac.quota,fac.quota)
        tail = ','*(n_students - len(fac.prefs))
        try:
            liststu = ','.join([stu.name for stu in fac.prefs]) + tail
        except TypeError:
            liststu = tail
        f.write(fac.name+','+str_fac+liststu+'\n')

# create a file for students' preferences
with open(file_out_stu, 'wt') as f:
    f.write('id,type' + ','*(n_faculty+1) + '\n')
    for stu in students:
        tail = ','*(n_faculty - len(stu.prefs))
        try:
            listfac = ','.join([fac.name for fac in stu.prefs]) + tail
        except TypeError:
            listfac = tail
        f.write(stu.name+',,'+listfac+'\n')
